@@ -20,11 +20,14 @@ import {
   fromIndicesArray,
   VennDiagramProps,
   renderVennDiagram,
+  renderKarnaughMap,
+  KarnaughMapProps,
   categoricalAddon,
   createVennJSAdapter,
+  ISets,
 } from '@upsetjs/bundle';
 import { layout } from '@upsetjs/venn.js';
-import { fixCombinations, fixSets, resolveSet, IElem } from './utils';
+import { fixCombinations, fixSets, resolveSet, IElem, fromExpression } from './utils';
 
 const adapter = createVennJSAdapter(layout);
 
@@ -110,8 +113,8 @@ export class UpSetView extends DOMWidgetView {
     this.model.on('change', this.changed_prop, this);
   };
 
-  private stateToProps(): Partial<UpSetProps<any> & VennDiagramProps<any>> {
-    const props: Partial<UpSetProps<any> & VennDiagramProps<any>> = {};
+  private stateToProps(): Partial<UpSetProps<any> & VennDiagramProps<any> & KarnaughMapProps<any>> {
+    const props: Partial<UpSetProps<any> & VennDiagramProps<any> & KarnaughMapProps<any>> = {};
     const state = this.model.get_state(true);
 
     const toCamelCase = (v: string) => v.replace(/([-_]\w)/g, (g) => g[1].toUpperCase());
@@ -221,11 +224,17 @@ export class UpSetView extends DOMWidgetView {
       props.sets = fixSets(delta.sets, this.elems);
     }
     if (delta.combinations != null) {
-      const c = fixCombinations(delta.combinations, props.sets, this.elems);
-      if (c == null) {
-        delete props.combinations;
+      if (this.model.get('_expression_data')) {
+        const r = fromExpression(delta.combinations);
+        props.combinations = r.combinations as ISetCombinations<IElem>;
+        props.sets = r.sets as ISets<IElem>;
       } else {
-        props.combinations = c;
+        const c = fixCombinations(delta.combinations, props.sets, this.elems);
+        if (c == null) {
+          delete props.combinations;
+        } else {
+          props.combinations = c;
+        }
       }
     }
     if (delta.selection) {
@@ -277,33 +286,31 @@ export class UpSetView extends DOMWidgetView {
     const p = Object.assign({}, this.props);
     const renderMode = this.model.get('_render_mode');
 
+    const renderComponent = () => {
+      if (renderMode === 'venn') {
+        delete p.layout;
+        renderVennDiagram(this.el, p);
+      } else if (renderMode === 'euler') {
+        p.layout = adapter;
+        renderVennDiagram(this.el, p);
+      } else if (renderMode === 'kmap') {
+        renderKarnaughMap(this.el, p);
+      } else {
+        render(this.el, p);
+      }
+    };
+
     if (!bb.width || !bb.height) {
       requestAnimationFrame(() => {
         const bb2 = this.el.getBoundingClientRect();
         p.width = bb2.width || 600;
         p.height = bb2.height || 400;
-        if (renderMode === 'venn') {
-          delete p.layout;
-          renderVennDiagram(this.el, p);
-        } else if (renderMode === 'euler') {
-          p.layout = adapter;
-          renderVennDiagram(this.el, p);
-        } else {
-          render(this.el, p);
-        }
+        renderComponent();
       });
       return;
     }
     p.width = bb.width;
     p.height = bb.height;
-    if (renderMode === 'venn') {
-      delete p.layout;
-      renderVennDiagram(this.el, p);
-    } else if (renderMode === 'euler') {
-      p.layout = adapter;
-      renderVennDiagram(this.el, p);
-    } else {
-      render(this.el, p);
-    }
+    renderComponent();
   }
 }
