@@ -31,6 +31,7 @@ from ._model import (
     T,
     UpSetQuery,
     UpSetSet,
+    UpSetSetType,
     UpSetSetCombination,
     UpSetSetComposite,
     UpSetSetIntersection,
@@ -135,6 +136,30 @@ def _to_query_list(arr: t.List[UpSetQuery], model: "UpSetJSWidget"):
     return [_to_query(q) for q in arr]
 
 
+def _create_combination(
+    cs_name: str,
+    combination_type: UpSetSetType,
+    count: int,
+    sets: t.FrozenSet[UpSetSet],
+    color_lookup: t.Mapping[str, str],
+) -> UpSetSetCombination:
+    if combination_type == UpSetSetType.DISTINCT_INTERSECTION:
+        return UpSetSetDistinctIntersection[T](
+            cs_name, sets=sets, color=color_lookup.get(cs_name), cardinality=count,
+        )
+    if combination_type == UpSetSetType.INTERSECTION:
+        return UpSetSetIntersection[T](
+            cs_name, sets=sets, color=color_lookup.get(cs_name), cardinality=count,
+        )
+    if combination_type == UpSetSetType.UNION:
+        return UpSetSetUnion[T](
+            cs_name, sets=sets, color=color_lookup.get(cs_name), cardinality=count,
+        )
+    return UpSetSetComposite[T](
+        cs_name, sets=sets, color=color_lookup.get(cs_name), cardinality=count,
+    )
+
+
 class UpSetJSBaseWidget(ValueWidget, DOMWidget, t.Generic[T]):
     """UpSet.js Base Widget
     """
@@ -198,6 +223,8 @@ class UpSetJSBaseWidget(ValueWidget, DOMWidget, t.Generic[T]):
     title: str = Unicode(None, allow_none=True).tag(sync=True)
     description: str = Unicode(None, allow_none=True).tag(sync=True)
 
+    _expression_data: bool = Bool(None, allow_none=True).tag(sync=True)
+
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
         self.observe(self._sync_value, "value")
@@ -212,6 +239,8 @@ class UpSetJSBaseWidget(ValueWidget, DOMWidget, t.Generic[T]):
         clone.sets = list(self.sets)
         clone.combinations = list(self.combinations)
         clone.queries = list(self.queries)
+        # pylint: disable=protected-access
+        clone._expression_data = self._expression_data
 
         clone.value = self.value
         clone.selection = self.selection
@@ -384,7 +413,7 @@ class UpSetJSBaseWidget(ValueWidget, DOMWidget, t.Generic[T]):
 
     def _from_dict(
         self,
-        sets: t.Dict[str, t.Sequence[T]],
+        sets: t.Mapping[str, t.Sequence[T]],
         order_by: str = "cardinality",
         limit: t.Optional[int] = None,
         colors: t.Mapping[str, str] = None,
@@ -397,6 +426,7 @@ class UpSetJSBaseWidget(ValueWidget, DOMWidget, t.Generic[T]):
             elems.update(set_elems)
         self.elems = sorted(elems)
         self.elem_to_index = {e: i for i, e in enumerate(self.elems)}
+        self._expression_data = False
         color_lookup = colors or {}
 
         base_sets: t.List[UpSetSet[T]] = [
@@ -409,6 +439,69 @@ class UpSetJSBaseWidget(ValueWidget, DOMWidget, t.Generic[T]):
 
         return base_sets if limit is None else self.sets
 
+    def from_expression(
+        self,
+        combinations: t.Mapping[str, int],
+        combination_type: UpSetSetType = UpSetSetType.INTERSECTION,
+        symbol: str = "&",
+        order_by: t.Union[str, t.Sequence[str]] = "cardinality",
+        set_order_by: str = "cardinality",
+        colors: t.Mapping[str, str] = None,
+    ) -> "UpSetJSBaseWidget":
+        """
+        generates the list of sets from a dict
+        """
+        self.elems = []
+        self.elem_to_index = {}
+        self._expression_data = True
+        color_lookup = colors or {}
+
+        base_sets: t.Dict[str, UpSetSet[T]] = {}
+        set_intersections: t.List[UpSetSetCombination[T]] = []
+
+        for cs_name, count in combinations.items():
+            contained_sets: t.List[UpSetSet[T]] = []
+
+            for set_name in cs_name.split(symbol):
+                if set_name not in base_sets:
+                    base_sets[set_name] = UpSetSet[T](
+                        set_name,
+                        elems=None,
+                        color=color_lookup.get(set_name),
+                        cardinality=0,
+                    )
+                contained_sets.append(base_sets[set_name])
+
+            if combination_type == UpSetSetType.DISTINCT_INTERSECTION:
+                for s_obj in contained_sets:
+                    s_obj.cardinality += count
+            elif len(contained_sets) == 1:
+                contained_sets[0].cardinality = count
+            elif combination_type == UpSetSetType.INTERSECTION:
+                for s_obj in contained_sets:
+                    s_obj.cardinality = max(s_obj.cardinality, count)
+            elif combination_type == UpSetSetType.UNION:
+                for s_obj in contained_sets:
+                    s_obj.cardinality = min(s_obj.cardinality, count)
+
+            set_intersections.append(
+                _create_combination(
+                    cs_name,
+                    combination_type,
+                    count,
+                    frozenset(contained_sets),
+                    color_lookup,
+                )
+            )
+
+        self.clear_queries()
+        self.selection = None
+        self.sets = _sort_sets(list(base_sets.values()), set_order_by, None)
+        self.combinations = _sort_combinations(
+            set_intersections, self.sets, order_by, None
+        )
+        return self
+
     def _from_dataframe(
         self,
         data_frame: t.Any,
@@ -422,6 +515,7 @@ class UpSetJSBaseWidget(ValueWidget, DOMWidget, t.Generic[T]):
         """
         self.elems = sorted(data_frame.index)
         self.elem_to_index = {e: i for i, e in enumerate(self.elems)}
+        self._expression_data = False
         color_lookup = colors or {}
 
         def to_set(name: str, series):
@@ -509,7 +603,7 @@ class UpSetJSWidget(UpSetJSBaseWidget, t.Generic[T]):
 
     def from_dict(
         self,
-        sets: t.Dict[str, t.Sequence[T]],
+        sets: t.Mapping[str, t.Sequence[T]],
         order_by: str = "cardinality",
         limit: t.Optional[int] = None,
         colors: t.Mapping[str, str] = None,
@@ -528,7 +622,7 @@ class UpSetJSWidget(UpSetJSBaseWidget, t.Generic[T]):
         order_by: str = "cardinality",
         limit: t.Optional[int] = None,
         colors: t.Mapping[str, str] = None,
-    ):
+    ) -> "UpSetJSWidget":
         """
         generates the list of sets from a dataframe
         """
@@ -565,7 +659,7 @@ class UpSetJSWidget(UpSetJSBaseWidget, t.Generic[T]):
         base_sets: t.List[UpSetSet[T]],
         order_by: t.Union[str, t.Sequence[str]] = "cardinality",
         colors: t.Mapping[str, str] = None,
-    ):
+    ) -> "UpSetJSWidget":
         """
         customize the generation of the sets
         """
@@ -586,7 +680,7 @@ class UpSetJSWidget(UpSetJSBaseWidget, t.Generic[T]):
         order_by: t.Union[str, t.Sequence[str]] = "cardinality",
         limit: t.Optional[int] = None,
         colors: t.Mapping[str, str] = None,
-    ):
+    ) -> "UpSetJSWidget":
         """
         customize the generation of the sets
         """
@@ -607,7 +701,7 @@ class UpSetJSWidget(UpSetJSBaseWidget, t.Generic[T]):
         order_by: t.Union[str, t.Sequence[str]] = "cardinality",
         limit: t.Optional[int] = None,
         colors: t.Mapping[str, str] = None,
-    ):
+    ) -> "UpSetJSWidget":
         """
         customize the generation of the sets
         """
@@ -628,7 +722,7 @@ class UpSetJSWidget(UpSetJSBaseWidget, t.Generic[T]):
         order_by: t.Union[str, t.Sequence[str]] = "cardinality",
         limit: t.Optional[int] = None,
         colors: t.Mapping[str, str] = None,
-    ):
+    ) -> "UpSetJSWidget":
         """
         customize the generation of the sets
         """
@@ -639,11 +733,12 @@ class UpSetJSWidget(UpSetJSBaseWidget, t.Generic[T]):
         self.combinations = _sort_combinations(set_unions, self.sets, order_by, limit)
         return self
 
-    def clear_attributes(self):
+    def clear_attributes(self) -> "UpSetJSWidget":
         """
         deletes the list of attributes
         """
         self.attrs = []
+        return self
 
     def append_numeric_attribute(
         self,
@@ -701,7 +796,7 @@ class UpSetJSVennDiagramWidget(UpSetJSBaseWidget, t.Generic[T]):
 
     def from_dict(
         self,
-        sets: t.Dict[str, t.Sequence[T]],
+        sets: t.Mapping[str, t.Sequence[T]],
         order_by: str = "cardinality",
         limit: t.Optional[int] = None,
         colors: t.Mapping[str, str] = None,
@@ -719,7 +814,7 @@ class UpSetJSVennDiagramWidget(UpSetJSBaseWidget, t.Generic[T]):
         order_by: str = "cardinality",
         limit: t.Optional[int] = None,
         colors: t.Mapping[str, str] = None,
-    ):
+    ) -> "UpSetJSVennDiagramWidget":
         """
         generates the list of sets from a dataframe
         """
@@ -731,7 +826,7 @@ class UpSetJSVennDiagramWidget(UpSetJSBaseWidget, t.Generic[T]):
 
     def _generate_distinct_intersections(
         self, base_sets: t.List[UpSetSet[T]], colors: t.Mapping[str, str] = None,
-    ):
+    ) -> "UpSetJSVennDiagramWidget":
         set_intersections = generate_distinct_intersections(
             base_sets, 1, None, True, self.elems, colors=colors
         )
@@ -750,3 +845,83 @@ class UpSetJSEulerDiagramWidget(UpSetJSVennDiagramWidget, t.Generic[T]):
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
         self._render_mode = "euler"
+
+    def copy(self) -> "UpSetJSEulerDiagramWidget":
+        """
+        returns a copy of itself
+        """
+        clone = UpSetJSEulerDiagramWidget[T]()  # pylint: disable=unsubscriptable-object
+        self._base_copy(clone)
+        clone.value_text_color = self.value_text_color
+        clone.stroke_color = self.stroke_color
+
+        return clone
+
+
+@register
+class UpSetJSKarnaughMapWidget(UpSetJSBaseWidget, t.Generic[T]):
+    """UpSet.js Karnaugh Map Widget
+    """
+
+    _render_mode = Unicode("kmap").tag(sync=True)
+
+    bar_padding: float = Float(None, allow_none=True).tag(sync=True)
+    """
+    padding argument for scaleBand (0..1)
+    """
+    stroke_color: str = Color(None, allow_none=True).tag(sync=True)
+    numeric_scale: str = Enum(("linear", "log"), default_value="linear").tag(sync=True)
+
+    def copy(self) -> "UpSetJSKarnaughMapWidget":
+        """
+        returns a copy of itself
+        """
+        clone = UpSetJSKarnaughMapWidget[T]()  # pylint: disable=unsubscriptable-object
+        self._base_copy(clone)
+        clone.bar_padding = self.bar_padding
+        clone.stroke_color = self.stroke_color
+        clone.numeric_scale = self.numeric_scale
+
+        return clone
+
+    def from_dict(
+        self,
+        sets: t.Mapping[str, t.Sequence[T]],
+        order_by: str = "cardinality",
+        limit: t.Optional[int] = None,
+        colors: t.Mapping[str, str] = None,
+    ) -> "UpSetJSKarnaughMapWidget":
+        """
+        generates the list of sets from a dict
+        """
+        base_sets = super()._from_dict(sets, order_by, limit, colors)
+        return self._generate_distinct_intersections(base_sets, colors=colors)
+
+    def from_dataframe(
+        self,
+        data_frame: t.Any,
+        attributes: t.Union[t.Sequence[str], t.Any, None] = None,
+        order_by: str = "cardinality",
+        limit: t.Optional[int] = None,
+        colors: t.Mapping[str, str] = None,
+    ) -> "UpSetJSKarnaughMapWidget":
+        """
+        generates the list of sets from a dataframe
+        """
+        base_sets = super()._from_dataframe(
+            data_frame, attributes, order_by, limit, colors
+        )
+
+        return self._generate_distinct_intersections(base_sets, colors=colors)
+
+    def _generate_distinct_intersections(
+        self, base_sets: t.List[UpSetSet[T]], colors: t.Mapping[str, str] = None,
+    ) -> "UpSetJSKarnaughMapWidget":
+        set_intersections = generate_distinct_intersections(
+            base_sets, 1, None, True, self.elems, colors=colors
+        )
+
+        self.combinations = _sort_combinations(
+            set_intersections, self.sets, ["degree", "group"]
+        )
+        return self
